<?php

namespace WebsiteBundle\Entity;

use Bazookas\CommonBundle\Entity\Base\BaseCMSUser;
use Bazookas\CommonBundle\Entity\Base\BaseUser;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Security\Core\Authorization\AuthorizationChecker;
use Symfony\Component\Validator\Constraints as Assert;
use WebsiteBundle\Security\Roles;

/**
 * @ORM\Entity(repositoryClass="WebsiteBundle\Repository\ParticipantRepository")
 * @ORM\HasLifecycleCallbacks
 */
class Participant extends BaseUser {

  /**
   * @ORM\Column(type="string")
   * @var string
   * @Assert\NotBlank(groups={"addOrEdit"})
   */
  private $name;

  /**
   * @ORM\Column(type="string")
   * @var string
   * @Assert\NotBlank(groups={"addOrEdit"})
   */
  private $surname;

  /**
   * @ORM\Column(type="string")
   * @var string
   * @Assert\Email(groups={"addOrEdit"})
   */
  private $email;

  /**
   * @ORM\Column(type="string", nullable=true)
   * @var string
   */
  private $phoneNumber;

  /**
   * @ORM\Column(type="boolean")
   * @var boolean
   */
  private $agreesToTermsAndConditions = false;

  /**
   * @ORM\ManyToOne(targetEntity="Experiment", inversedBy="participants")
   * @var Experiment
   * @Assert\NotBlank(groups={"addOrEdit"})
   */
  private $experiment;

  /**
   * @ORM\OneToMany(targetEntity="TestSession", mappedBy="participant")
   * @var ArrayCollection
   */
  private $sessions;


  /**
   * @ORM\OneToMany(targetEntity="PsychEducationAnswer", mappedBy="participant")
   */
  private $psychEducationAnswers;

  /**
   * @ORM\Column(type="boolean")
   * @var boolean
   */
  private $active = true;

  /**
   * @ORM\ManyToOne(targetEntity="WebsiteBundle\Entity\ExperimentCondition", inversedBy="participants")
   * @var ExperimentCondition
   */
  private $condition;

  /**
   * @ORM\Column(type="boolean")
   * @var bool
   */
  private $completedPsychEducation = false;

  /**
   * An unmapped field to be used in forms when assigning or changing the user's password.
   * @var string
   */
  private $plainPassword;

  /**
   * @ORM\Column(type="datetime", nullable=true)
   * @var \DateTime|null
   */
  private $activeOverrideExpirationDate;

  /**
   * @ORM\Column(type="boolean")
   * @var bool
   * @Assert\Type(type="bool", groups={"skip"})
   */
  private $skipTutorial = false;

  /**
   * @ORM\Column(type="string", nullable=true)
   * @var string
   */
  private $forgotPasswordHash;

  /**
   * @ORM\Column(type="boolean")
   * @var bool
   */
  private $completedAllSessions = false;

  /**
   * @ORM\OneToMany(targetEntity="WebsiteBundle\Entity\UnlockedQuote", mappedBy="participant")
   * @var ArrayCollection
   */
  private $unlockedQuotes;

  /**
   * @ORM\Column(type="boolean")
   * @var boolean
   */
  private $consented = false;

  /**
   * @ORM\Column(type="datetime", nullable=true)
   * @var \DateTime
   */
  private $consentedOn;

  public function __construct($username = '', $password = '', $salt = '', array $roles = [ Roles::ROLE_PARTICIPANT ]) {
    parent::__construct($username, $password, $salt, $roles);
    $this->sessions = new ArrayCollection();
    $this->unlockedQuotes = new ArrayCollection();
  }

  public function getName()
  {
    return $this->name;
  }

  public function setName($name)
  {
    $this->name = $name;

    return $this;
  }

  public function getSurname()
  {
    return $this->surname;
  }

  public function setSurname($surname)
  {
    $this->surname = $surname;

    return $this;
  }

  public function getEmail()
  {
    return $this->email;
  }

  public function setEmail($email)
  {
    $this->email = $email;
    $this->setUsername($email);

    return $this;
  }

  public function getPhoneNumber()
  {
    return $this->phoneNumber;
  }

  public function setPhoneNumber($phoneNumber)
  {
    $this->phoneNumber = $phoneNumber;

    return $this;
  }

  public function isAgreesToTermsAndConditions()
  {
    return $this->agreesToTermsAndConditions;
  }

  public function setAgreesToTermsAndConditions($agreesToTermsAndConditions)
  {
    $this->agreesToTermsAndConditions = $agreesToTermsAndConditions;

    return $this;
  }

  public function getExperiment()
  {
    return $this->experiment;
  }

  public function setExperiment(Experiment $experiment = null)
  {
    if (!empty($this->experiment)) {
      $this->experiment->setParticipantCount($this->experiment->getParticipantCount() - 1);
    }

    $experiment->setParticipantCount($experiment->getParticipantCount() + 1);
    $this->experiment = $experiment;

    return $this;
  }

  public function getSessions()
  {
    return $this->sessions;
  }

  public function setSessions($sessions)
  {
    $this->sessions = $sessions;

    return $this;
  }

  public function getPsychEducationAnswers()
  {
    return $this->psychEducationAnswers;
  }

  public function setPsychEducationAnswers($psychEducationAnswers)
  {
    $this->psychEducationAnswers = $psychEducationAnswers;

    return $this;
  }

  public function isActive()
  {
    return $this->active;
  }

  public function setActive($active)
  {
    if (!$this->active && $active) {
      $expirationDate = new \DateTime('now', new \DateTimeZone('UTC'));
      $expirationDate->modify('+48 hours');
      $this->activeOverrideExpirationDate = $expirationDate;
    }

    $this->active = $active;

    return $this;
  }

  /**
   * @return ExperimentCondition
   */
  public function getCondition()
  {
    return $this->condition;
  }

  /**
   * @param ExperimentCondition $condition
   * @return $this
   */
  public function setCondition(ExperimentCondition $condition)
  {
    $this->condition = $condition;

    return $this;
  }

  public function hasCompletedPsychEducation()
  {
    return $this->completedPsychEducation;
  }

  public function setCompletedPsychEducation($completedPsychEducation)
  {
    $this->completedPsychEducation = $completedPsychEducation;

    return $this;
  }

  public function getPlainPassword()
  {
    return $this->plainPassword;
  }

  public function setPlainPassword($plainPassword)
  {
    $this->plainPassword = $plainPassword;

    return $this;
  }

  public function getActiveOverrideExpirationDate()
  {
    return $this->activeOverrideExpirationDate;
  }

  public function setActiveOverrideExpirationDate(\DateTime $activeOverrideExpirationDate)
  {
    $this->activeOverrideExpirationDate = $activeOverrideExpirationDate;

    return $this;
  }

  /**
   * @return bool
   */
  public function isSkipTutorial()
  {
    return $this->skipTutorial;
  }

  /**
   * @param bool $skipTutorial
   * @return Participant
   */
  public function setSkipTutorial($skipTutorial)
  {
    $this->skipTutorial = $skipTutorial;

    return $this;
  }

  /**
   * @return mixed
   */
  public function getForgotPasswordHash()
  {
    return $this->forgotPasswordHash;
  }

  /**
   * @param mixed $forgotPasswordHash
   * @return Participant
   */
  public function setForgotPasswordHash($forgotPasswordHash)
  {
    $this->forgotPasswordHash = $forgotPasswordHash;

    return $this;
  }

  /**
   * @return bool
   */
  public function hasCompletedAllSessions()
  {
    return $this->completedAllSessions;
  }

  /**
   * @param bool $completedAllSessions
   * @return Participant
   */
  public function setCompletedAllSessions($completedAllSessions)
  {
    $this->completedAllSessions = $completedAllSessions;

    return $this;
  }

  /**
   * @return ArrayCollection
   */
  public function getUnlockedQuotes()
  {
    return $this->unlockedQuotes;
  }

  /**
   * @param ArrayCollection $unlockedQuotes
   * @return Participant
   */
  public function setUnlockedQuotes($unlockedQuotes)
  {
    $this->unlockedQuotes = $unlockedQuotes;

    return $this;
  }

  /**
   * @param UnlockedQuote $quote
   * @return $this
   */
  public function addUnlockedQuote(UnlockedQuote $quote) {
    $this->unlockedQuotes->add($quote);
    return $this;
  }

  /**
   * @ORM\PrePersist
   */
  public function preFormPersist()
  {
    $this->created = new \DateTime('now', new \DateTimeZone('UTC'));
    $this->setUsername($this->getEmail());
  }

  /**
   * Override of AccessControlTrait
   * @param AuthorizationChecker $auth
   * @param BaseCMSUser $user
   * @return bool
   */
  public function canBulkEdit(AuthorizationChecker $auth, BaseCMSUser $user)
  {
    if ($this->sessions->isEmpty()) {
      return parent::canBulkEdit($auth, $user);
    } else {
      return false;
    }
  }

  /**
   * @return string
   */
  public function getRoleRequiredForView() {
    return Roles::ROLE_CMS_USER;
  }

  public function toHumanReadable($locale = 'nl')
  {
    if ($locale == 'nl') {
      return 'Deelnemer ' . $this->id . ' is reeds gekoppeld aan een lopend experiment.';
    } else {
      return parent::toHumanReadable($locale);
    }
  }

  public function canDelete(AuthorizationChecker $auth, BaseCMSUser $user) {
    return $this->sessions->isEmpty();
  }

  /**
   * @return bool
   */
  public function isConsented()
  {
    return $this->consented;
  }

  /**
   * @param bool $consented
   * @return $this
   */
  public function setConsented($consented)
  {
    $this->consented = $consented;

    return $this;
  }

  /**
   * @return \DateTime
   */
  public function getConsentedOn()
  {
    return $this->consentedOn;
  }

  /**
   * @param \DateTime $consentedOn
   * @return $this
   */
  public function setConsentedOn(\DateTime $consentedOn)
  {
    $this->consentedOn = $consentedOn;

    return $this;
  }

}
