<?php

namespace WebsiteBundle\Entity;

use Bazookas\CommonBundle\Entity\Base\BaseEntity;
use DateTime;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;
use WebsiteBundle\Security\Roles;

/**
 * @ORM\Entity(repositoryClass="WebsiteBundle\Repository\TestSessionRepository")
 * @ORM\HasLifecycleCallbacks
 */
class TestSession extends BaseEntity {

  //Min interval in milliseconds
  const MIN_INTERVAL = 1500;

  //Device constants
  const DEVICE_TABLET = 'TABLET';
  const DEVICE_PC = 'PC';

  //Color constants
  const BACKGROUND_COLOR_RED = 'bg-red';
  const BACKGROUND_COLOR_BLUE = 'bg-blue';
  const BACKGROUND_COLOR_YELLOW = 'bg-yellow';
  const BACKGROUND_COLOR_PURPLE = 'bg-purple';

  /**
   * @ORM\Column(type="integer")
   * @var integer
   * @Assert\NotBlank
   */
  private $number = 1;

  /**
   * @ORM\ManyToOne(targetEntity="Participant", inversedBy="sessions")
   */
  private $participant;

  /**
   * @ORM\ManyToOne(targetEntity="Experiment", inversedBy="sessions")
   */
  private $experiment;

  /**
   * @ORM\Column(type="datetime", nullable=true)
   * @var DateTime
   * @Assert\NotBlank
   */
  private $scheduled;

  /**
   * @ORM\Column(type="datetime", nullable=true)
   * @var DateTime
   * @Assert\NotBlank
   */
  private $availableFrom;

  /**
   * @ORM\Column(type="datetime", nullable=true)
   * @var DateTime
   * @Assert\NotBlank
   */
  private $completed;

  /**
   * @ORM\Column(type="integer")
   * @var integer (in seconds)
   * @Assert\NotBlank
   */
  private $duration;

  /**
   * @ORM\Column(type="integer", nullable=true)
   * @var integer
   * @Assert\NotBlank
   */
  private $trialCount;

  /**
   * @ORM\Column(type="integer", nullable=true)
   * @var integer
   * @Assert\NotBlank
   */
  private $correctCount;

  /**
   * @ORM\Column(type="decimal", precision=6, scale=2, nullable=true)
   * @var float
   * @Assert\NotBlank
   */
  private $avgConsecutiveCorrect;

  /**
   * @ORM\Column(type="decimal", precision=6, scale=2, nullable=true)
   * @var float
   * @Assert\NotBlank
   */
  private $avgConsecutiveWrong;

  /**
   * @ORM\Column(type="string", nullable=true)
   * @var string
   * @Assert\Choice(choices={"Tablet", "PC"})
   */
  private $device;

  /**
   * @ORM\Column(type="string", nullable=true)
   * @var string
   * @Assert\Choice(choices={"Mouse", "Touch"})
   */
  private $inputType;

  /**
   * @ORM\Column(type="decimal", precision=6, scale=2, nullable=true)
   * @var float
   * @Assert\NotBlank
   */
  private $meanReactionTime;

  /**
   * @ORM\Column(type="integer")
   * @var integer (in milliseconds)
   * @Assert\NotBlank
   */
  private $initialInterval;

  /**
   * @ORM\Column(type="integer", nullable=true)
   * @var integer (in milliseconds)
   * @Assert\NotBlank
   */
  private $minInterval;

  /**
   * @ORM\Column(type="integer", nullable=true)
   * @var integer (in milliseconds)
   * @Assert\NotBlank
   */
  private $maxInterval;

  /**
   * @ORM\Column(type="integer", nullable=true)
   * @var int (in milliseconds)
   * @Assert\NotBlank
   */
  private $medianInterval;

  /**
   * @ORM\OneToMany(targetEntity="TestAnswer", mappedBy="session")
   */
  private $answers;

  /**
   * @ORM\OneToMany(targetEntity="MoodAnswer", mappedBy="session")
   */
  private $moodAnswers;

  /**
   * @ORM\Column(type="string")
   * @var string
   * @Assert\Choice(choices={"PASAT", "APASAT", "CONTROL"})
   */
  private $type;

  /**
   * @ORM\Column(type="string")
   * @var string
   */
  private $gamificationLevel;

  /**
   * @ORM\Column(type="text", nullable=true)
   * @var string
   */
  private $comment;

  /**
   * @ORM\Column(type="boolean")
   * @var boolean
   */
  private $sentBeforeNotification = false;

  /**
   * @ORM\Column(type="boolean")
   * @var boolean
   */
  private $sentAfterNotification = false;

  /**
   * @ORM\Column(type="integer")
   * @var int
   */
  private $initialLevel = 1;

  /**
   * @ORM\Column(type="integer")
   * @var int
   */
  private $finalLevel = 1;

  /**
   * @ORM\Column(type="integer")
   * @var int
   */
  private $totalScore = 0;

  /**
   * @ORM\Column(type="string", nullable=true)
   * @var string
   * High gamification games have a random color assigned to them
   */
  private $color;

  public function __construct() {
    parent::__construct();
    $this->answers = new ArrayCollection();
    $this->moodAnswers = new ArrayCollection();
  }

  public function getNumber()
  {
    return $this->number;
  }

  public function setNumber($number)
  {
    $this->number = $number;

    return $this;
  }

  public function getParticipant()
  {
    return $this->participant;
  }

  public function setParticipant($participant)
  {
    $this->participant = $participant;

    return $this;
  }

  public function getExperiment()
  {
    return $this->experiment;
  }

  public function setExperiment($experiment)
  {
    $this->experiment = $experiment;

    return $this;
  }

  public function getScheduled()
  {
    return $this->scheduled;
  }

  public function setScheduled($scheduled)
  {
    $this->scheduled = $scheduled;

    return $this;
  }

  public function getAvailableFrom()
  {
    return $this->availableFrom;
  }

  public function setAvailableFrom($availableFrom)
  {
    $this->availableFrom = $availableFrom;

    return $this;
  }

  public function getCompleted()
  {
    return $this->completed;
  }

  public function setCompleted($completed)
  {
    $this->completed = $completed;

    return $this;
  }

  public function getDuration()
  {
    return $this->duration;
  }

  public function setDuration($duration)
  {
    $this->duration = $duration;

    return $this;
  }

  public function getTrialCount()
  {
    return $this->trialCount;
  }

  public function setTrialCount($trialCount)
  {
    $this->trialCount = $trialCount;

    return $this;
  }

  public function getCorrectCount()
  {
    return $this->correctCount;
  }

  public function setCorrectCount($correctCount)
  {
    $this->correctCount = $correctCount;

    return $this;
  }

  public function getAvgConsecutiveCorrect()
  {
    return $this->avgConsecutiveCorrect;
  }

  public function setAvgConsecutiveCorrect($avgConsecutiveCorrect)
  {
    $this->avgConsecutiveCorrect = $avgConsecutiveCorrect;

    return $this;
  }

  public function getAvgConsecutiveWrong()
  {
    return $this->avgConsecutiveWrong;
  }

  public function setAvgConsecutiveWrong($avgConsecutiveWrong)
  {
    $this->avgConsecutiveWrong = $avgConsecutiveWrong;

    return $this;
  }

  public function getDevice()
  {
    return $this->device;
  }

  public function setDevice($device)
  {
    $this->device = $device;

    return $this;
  }

  public function getInputType()
  {
    return $this->inputType;
  }

  public function setInputType($inputType)
  {
    $this->inputType = $inputType;

    return $this;
  }

  public function getMeanReactionTime()
  {
    return $this->meanReactionTime;
  }

  public function setMeanReactionTime($meanReactionTime)
  {
    $this->meanReactionTime = $meanReactionTime;

    return $this;
  }

  public function getInitialInterval()
  {
    return $this->initialInterval;
  }

  public function setInitialInterval($initialInterval)
  {
    $this->initialInterval = $initialInterval;

    return $this;
  }

  public function getMinInterval()
  {
    return $this->minInterval;
  }

  public function setMinInterval($minInterval)
  {
    $this->minInterval = $minInterval;

    return $this;
  }

  public function getMaxInterval()
  {
    return $this->maxInterval;
  }

  public function setMaxInterval($maxInterval)
  {
    $this->maxInterval = $maxInterval;

    return $this;
  }

  public function getMedianInterval()
  {
    return $this->medianInterval;
  }

  public function setMedianInterval($medianInterval)
  {
    $this->medianInterval = $medianInterval;

    return $this;
  }

  public function getAnswers()
  {
    return $this->answers;
  }

  public function setAnswers($answers)
  {
    $this->answers = $answers;

    return $this;
  }

  public function addAnswer(TestAnswer $answer) {
    $answer
      ->setSession($this)
      ->setParticipant($this->participant)
    ;
    $this->answers->add($answer);
    return $this;
  }

  public function getMoodAnswers()
  {
    return $this->moodAnswers;
  }

  public function setMoodAnswers($moodAnswers)
  {
    $this->moodAnswers = $moodAnswers;

    return $this;
  }

  public function getType()
  {
    return $this->type;
  }

  public function setType($type)
  {
    $this->type = $type;

    return $this;
  }

  /**
   * @return string
   */
  public function getGamificationLevel()
  {
    return $this->gamificationLevel;
  }

  /**
   * @param string $gamificationLevel
   * @return $this
   */
  public function setGamificationLevel($gamificationLevel)
  {
    $this->gamificationLevel = $gamificationLevel;

    return $this;
  }

  public function getComment()
  {
    return $this->comment;
  }

  public function setComment($comment)
  {
    $this->comment = $comment;

    return $this;
  }

  public function isSentBeforeNotification()
  {
    return $this->sentBeforeNotification;
  }

  public function setSentBeforeNotification($sentBeforeNotification)
  {
    $this->sentBeforeNotification = $sentBeforeNotification;

    return $this;
  }

  public function isSentAfterNotification()
  {
    return $this->sentAfterNotification;
  }

  public function setSentAfterNotification($sentAfterNotification)
  {
    $this->sentAfterNotification = $sentAfterNotification;

    return $this;
  }

  /**
   * @return int
   */
  public function getInitialLevel()
  {
    return $this->initialLevel;
  }

  /**
   * @param int $initialLevel
   * @return TestSession
   */
  public function setInitialLevel($initialLevel)
  {
    $this->initialLevel = $initialLevel;

    return $this;
  }

  /**
   * @return int
   */
  public function getFinalLevel()
  {
    return $this->finalLevel;
  }

  /**
   * @param int $finalLevel
   * @return TestSession
   */
  public function setFinalLevel($finalLevel)
  {
    $this->finalLevel = $finalLevel;

    return $this;
  }

  /**
   * @return int
   */
  public function getTotalScore()
  {
    return $this->totalScore;
  }

  /**
   * @param int $totalScore
   * @return $this
   */
  public function setTotalScore($totalScore)
  {
    $this->totalScore = $totalScore;

    return $this;
  }

  /**
   * @return string
   */
  public function getColor()
  {
    return $this->color;
  }

  /**
   * @param string $color
   * @return $this
   */
  public function setColor($color)
  {
    $this->color = $color;

    return $this;
  }

  /**
   * @inheritdoc
   */
  public function getRoleRequiredForView() {
    return Roles::ROLE_CMS_USER;
  }
}
