## Logging
### Introduction

This component creates a log of the actions taken by users for the defined entities.

### Getting Started
#### Configuration

First begin by creating your own audit log entity. This entity will be used to store all of the logs and can be any class as long as it implements the `Bazookas\AdminBundle\Entity\Interfaces\AuditLogInterface`. A base implementation has been provided in the AdminBundle; you can therefore create your class as follows:

```php
/**
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 */
class AuditLog extends BaseAuditLog {

}
```

Now that you've created your `AuditLog` entity you need to notify the admin bundle of its existence in order to activate the loggable functionality. This is done by defining the logClass in the admin bundle configuration in `config.yml`:

```yaml
bazookas_admin:
  audit:
    logging:
      logClass: WebsiteBundle\Entity\AuditLog
```

#### Usage with entities

Once the logging functionality has been activated it is necessary to define which entities need to be monitored for logging. This can be done by implementing the `Bazookas\AdminBundle\Entity\Interfaces\AuditableInterface` for the entity you wish to create an audit log. An example is shown below:

```php
class Manufacturer extends BaseEntity implements AuditableInterface {

  use AuditableTrait;
  
}
```

As you can see in the example above a default implementation of the required functions in `AuditableInterface` has been created in the `Bazookas\AdminBundle\Entity\Traits\AuditableTrait`.

That's it! Any changes to your entity will now be logged in your `AuditLog` entity.
