<?php

namespace Bazookas\CommonBundle\Exception\Base;

use Bazookas\CommonBundle\Exception\Interfaces\BaseExceptionInterface;
use Exception;

class BaseException extends \RuntimeException implements BaseExceptionInterface {
  /**
   * @var string
   */
  protected $message;

  /**
   * @var string
   */
  protected $name;

  /**
   * BaseException constructor redefined such that message isn't optional
   * @param string $message
   * @param int|string $name
   * @param Exception|null $previous
   */
  public function __construct(string $message, $name = BaseExceptionMap::GENERAL_ERROR, Exception $previous = null) {
    $this->message = $message;
    $this->name = $name;
    parent::__construct($message, 0, $previous);
  }

  /**
   * @inheritdoc
   */
  public function __toString() {
    // custom string representation of object
    return get_class($this) . ": {$this->message}\n in '".$this->getFile()."' at line ".$this->getLine().".";
  }

  /**
   * @param string|int $name
   * @return $this
   */
  public function setName($name) {
    $this->name = $name;
    return $this;
  }

  /**
   * @return string|int
   */
  public function getName() {
    if (!empty($this->name)) {
      return $this->name;
    }
    $reflectionClass = new \ReflectionClass($this);
    return $reflectionClass->getShortName();
  }
}
