<?php

namespace Tests\Bazookas\CommonBundle\Enums;

use Symfony\Bundle\FrameworkBundle\Test\KernelTestCase;
use Symfony\Component\DependencyInjection\Container;
use Bazookas\CommonBundle\Tests\Bazookas\CommonBundle\Enums\TestEnum;

class BaseEnumTest extends KernelTestCase
{

  /**
   * @var Container
   */
  private $container;

  public function setUp()
  {
    self::bootKernel();
    $this->container = self::$kernel->getContainer();
  }

  /**
   * Test the 'isValidName' function
   */
  public function testIsValidName() {
    $this->assertTrue(TestEnum::isValidName('TEST_A'));
    $this->assertTrue(TestEnum::isValidName('TEST_B'));
    $this->assertTrue(TestEnum::isValidName('TEST_C'));

    $this->assertFalse(TestEnum::isValidName(null));
    $this->assertFalse(TestEnum::isValidName('TEST_D'));
  }

  /**
   * Test the 'getConstants' function
   */
  public function testGetConstants() {
    $constants = TestEnum::getConstants();

    $this->assertTrue(count($constants) === 3);
    $this->assertTrue(isset($constants['TEST_A']));
    $this->assertTrue(isset($constants['TEST_B']));
    $this->assertTrue(isset($constants['TEST_C']));
  }

  /**
   * Test the 'isValidValue' function
   */
  public function testIsValidValue() {
    $this->assertTrue(TestEnum::isValidValue('TEST_A'));
    $this->assertTrue(TestEnum::isValidValue('TEST_B'));
    $this->assertTrue(TestEnum::isValidValue(3));

    $this->assertFalse(TestEnum::isValidValue(null));
    $this->assertFalse(TestEnum::isValidValue('TEST_C'));
    $this->assertFalse(TestEnum::isValidValue('TEST_D'));
    $this->assertFalse(TestEnum::isValidValue(3.01));
  }

  /**
   * Test the 'getNameForValue' function
   */
  public function testNameForValue() {
    $this->assertTrue('TEST_A' === TestEnum::getNameForValue('TEST_A'));
    $this->assertTrue('TEST_B' === TestEnum::getNameForValue('TEST_B'));
    $this->assertTrue('TEST_C' === TestEnum::getNameForValue(3));

    $this->assertFalse(TestEnum::getNameForValue(3.01));
    $this->assertFalse(TestEnum::getNameForValue('TEST_C'));
  }

  /**
   * Test the 'getAllowedValues; function
   */
  public function testGetAllowedValues() {
    $values = TestEnum::getAllowedValues();

    $this->assertTrue(count($values) === 3);
    $this->assertTrue(in_array('TEST_A', $values));
    $this->assertTrue(in_array('TEST_B', $values));
    $this->assertTrue(in_array(3, $values));
  }
}
