<?php

namespace Bazookas\CommonBundle\Utility;

use Bazookas\CommonBundle\Entity\Interfaces\EntityInterface;
use Doctrine\Common\Collections\AbstractLazyCollection;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Persistence\Proxy;

class FormattingUtility
{
  /**
   * This static array serves to temporarily store the object arrays by their object hash.
   * This prevents
   * @var array
   */
  private static $parsedObjects = [];

  /**
   * @param $entity
   * @return array
   */
  public static function entityToArray($entity) {
    FormattingUtility::$parsedObjects = [];

    $array = FormattingUtility::toArray($entity);

    return $array;
  }

  /**
   * @param $object
   * @return array|mixed
   */
  protected static function toArray($object) {
    if (is_object($object)) {
      $object = FormattingUtility::convertObject($object);
    } else if (is_array($object)) {
      foreach($object as &$arrayItem) {
        if (is_object($arrayItem)) {
          $arrayItem = FormattingUtility::convertObject($arrayItem);
        } else {
          $arrayItem = FormattingUtility::toArray($arrayItem);
        }
      }
    }

    return $object;
  }

  /**
   * @param $object
   * @return array|mixed
   */
  protected static function convertObject($object) {
    if ($object instanceof EntityInterface && !$object instanceof Proxy) {
      $objectHash = spl_object_hash($object);

      if (array_key_exists('object-hash-'.$objectHash, FormattingUtility::$parsedObjects)) {
        return FormattingUtility::$parsedObjects['object-hash-'.$objectHash];
      } else {
        FormattingUtility::$parsedObjects['object-hash-'.$objectHash] = 'object-hash-'.$objectHash;
      }

      $array = FormattingUtility::convertObjectToArray($object);
      FormattingUtility::$parsedObjects['object-hash-'.$objectHash] = $array;
    } else if ($object instanceof ArrayCollection || ($object instanceof AbstractLazyCollection && $object->isInitialized())) {
      $array = $object->toArray();
      $array = FormattingUtility::toArray($array);
    } else {
      return $object;
    }

    return $array;
  }

  /**
   * @param $object
   * @return array
   */
  protected static function convertObjectToArray($object) {
    try {
      $refClass = new \ReflectionClass($object);
    } catch(\ReflectionException $e) {
      return $object;
    }

    $properties = $refClass->getProperties();

    $data = [];
    foreach($properties as $property) {
      $propertyName = $property->name;
      $functionName = 'get'.ucfirst($propertyName);

      if ($refClass->hasMethod($functionName)) {
        $value = $object->{$functionName}();

        if (is_object($value)) {
          $value = FormattingUtility::convertObject($value);
        }
        else if (is_array($value)) {
          $value = FormattingUtility::toArray($value);
        }

        $data[$propertyName] = $value;
      }
    }

    return $data;
  }

  /**
   * @param $hash
   * @return bool|mixed
   */
  public static function getObjectByHash($hash) {
    return isset(FormattingUtility::$parsedObjects[$hash]) ? FormattingUtility::$parsedObjects[$hash] : false;
  }

  /**
   * @return array
   */
  public static function getObjectHashMap(){
    return FormattingUtility::$parsedObjects;
  }

  /**
   * Clear the object hash map
   */
  public static function clearObjectHashMap() {
    FormattingUtility::$parsedObjects = [];
  }
}
