<?php

namespace Bazookas\ExportBundle\Entity;

use Bazookas\CommonBundle\Entity\Base\BaseEntity;
use Bazookas\ExportBundle\Enums\CSVDelimiter;
use Bazookas\ExportBundle\Enums\CSVEnclosure;
use Bazookas\ExportBundle\Enums\ImportFileType;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Symfony\Component\Validator\Constraints as Assert;

class GenericFileEntity extends BaseEntity {

  /**
   * @var UploadedFile
   * @Assert\File(
   *   mimeTypes = {"text/csv", "text/plain"},
   *   maxSize="10M"
   * )
   */
  private $file;

  /**
   * @var string
   */
  private $fileName;

  /**
   * @var string
   */
  private $originalFileName;

  /**
   * @var string
   */
  private $path;

  /**
   * @var string
   */
  private $fileType = ImportFileType::FILE_TYPE_CSV;

  /**
   * @var string
   */
  private $delimiter = CSVDelimiter::COMMA;

  /**
   * @var string
   */
  private $enclosure = CSVEnclosure::NEW_LINE;

  /**
   * @return UploadedFile
   */
  public function getFile()
  {
    return $this->file;
  }

  /**
   * @param UploadedFile $file
   * @return $this
   */
  public function setFile(UploadedFile $file)
  {
    $this->file = $file;

    return $this;
  }

  /**
   * @return string
   */
  public function getFileName()
  {
    return $this->fileName;
  }

  /**
   * @param string $fileName
   * @return $this
   */
  public function setFileName(string $fileName)
  {
    $this->fileName = $fileName;

    return $this;
  }

  /**
   * @return string
   */
  public function getOriginalFileName()
  {
    return $this->originalFileName;
  }

  /**
   * @param string $originalFileName
   * @return $this
   */
  public function setOriginalFileName(string $originalFileName)
  {
    $this->originalFileName = $originalFileName;

    return $this;
  }

  /**
   * @return string
   */
  public function getPath()
  {
    return $this->path;
  }

  /**
   * @param string $path
   * @return $this
   */
  public function setPath(string $path)
  {
    $this->path = $path;

    return $this;
  }

  /**
   * @return string
   */
  public function getFileType()
  {
    return $this->fileType;
  }

  /**
   * @param string $fileType
   * @return $this
   */
  public function setFileType(string $fileType = null)
  {
    if (is_null($fileType)){
      $fileType = ImportFileType::FILE_TYPE_CSV;
    }

    $this->fileType = $fileType;

    return $this;
  }

  /**
   * @return string
   */
  public function getDelimiter()
  {
    return $this->delimiter;
  }

  /**
   * @param string $delimiter
   * @return $this
   */
  public function setDelimiter(string $delimiter = null)
  {
    if (is_null($delimiter)){
      $delimiter = CSVDelimiter::SEMI_COLON;
    }

    $this->delimiter = $delimiter;

    return $this;
  }

  /**
   * @return string
   */
  public function getEnclosure()
  {
    return $this->enclosure;
  }

  /**
   * @param string $enclosure
   * @return $this
   */
  public function setEnclosure(string $enclosure = null)
  {
    if (is_null($enclosure)){
      $enclosure = CSVEnclosure::CARRIAGE_RETURN;
    }

    $this->enclosure = $enclosure;

    return $this;
  }

}
