<?php

namespace Bazookas\ExportBundle\Services\Data;

use Bazookas\CronBundle\Entity\Cron;
use Bazookas\CronBundle\Services\Base\BaseCronService;
use Symfony\Bundle\TwigBundle\TwigEngine;
use Symfony\Component\Translation\TranslatorInterface;

class ExportCronService extends BaseCronService {

  public function __construct(\Swift_Mailer $mailer, TwigEngine $templateEngine, TranslatorInterface $translator) {
    parent::__construct($mailer, $templateEngine, $translator);
  }

  public function onFailure(Cron $cron, array $options) {
    $to = $cron->getEmail();
    $subject = $this->getOnFailureSubject($cron, $options);
    $plainText = $this->getOnFailurePlainText($cron, $options);
    $html = $this->getOnFailureTemplate($cron, $options);
    $this->sendMail($to, $subject, $plainText, $html);
  }

  public function onSuccess(Cron $cron, array $options) {
    $to = $cron->getEmail();
    $subject = $this->getOnSuccessSubject($cron, $options);
    $plainText = $this->getOnSuccessPlainText($cron, $options);
    $html = $this->getOnSuccessTemplate($cron, $options);
    $this->sendMail($to, $subject, $plainText, $html);
  }

  protected function getOnFailureSubject(Cron $cron, $options) {
    return $this->translator->trans('admin.mail.exportFailed', [], 'admin');
  }

  protected function getOnFailurePlainText(Cron $cron, $options) {
    return
      "Greetings user!\n\n" .
      "It has come to our attention that the export you requested on " . date('D, d/m/Y ') . " at " . date('H:i:s') . " has failed to complete.\n\n" .
      "A possible cause is that the data you requested was too much for the server to process\n\n" .
      "We apologize for the inconvenience"
    ;
  }

  protected function getOnFailureTemplate(Cron $cron, $options) {
    return $this->templateEngine->render('BazookasExportBundle:Emails:onFailureMail.html.twig');
  }

  protected function getOnSuccessSubject(Cron $cron, $options) {
    return $this->translator->trans('admin.mail.exportSucceeded', [], 'admin');
  }

  protected function getOnSuccessPlainText(Cron $cron, $options) {
    $link = $cron->getEmailParameters()['filePath'];
    return
      "Greetings user!\n\n" .
      "The export you requested on " . date('D, d/m/Y ') . " at " . date('H:i:s') . " is now available for download.\n\n" .
      "To download your export, please use the following link:\n\n" . $link
    ;
  }

  protected function getOnSuccessTemplate(Cron $cron, $options) {
    $link = $cron->getEmailParameters()['filePath'];
    return $this->templateEngine->render('BazookasExportBundle:Emails:onSuccessMail.html.twig', array(
      'downloadLink' => $link,
    ));
  }
}
