<?php

namespace Bazookas\MediaBundle\Form\Type;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Bazookas\MediaBundle\Repository\Base\BaseMediaRepository;
use Symfony\Component\Form\CallbackTransformer;
use Symfony\Component\Security\Core\Authorization\AuthorizationChecker;

class MediaFolderType extends AbstractType
{
  protected $repository;
  private $auth;

  public function __construct(BaseMediaRepository $repository, AuthorizationChecker $auth) {
    $this->repository = $repository;
    $this->auth = $auth;
  }

  public function configureOptions(OptionsResolver $resolver) {
    $resolver->setDefaults(array(
      'label' => 'media.entities.mediaFolder.name',
      'choices' => $this->mediaFolderAsChoiceArray()
    ));
  }

  /**
  * @param FormBuilderInterface $builder
  * @param array $options
  */
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $builder
      ->addModelTransformer(new CallbackTransformer(
        function ($originalValue) {
          //transform original value into form field format
          if (!empty($originalValue)) {
            return $originalValue->getId();
          } else {
            return null;
          }
        },
        function ($formFieldValue) {
          //convert from form field format to original format
          if (!empty($formFieldValue)) {
            $folder = $this->repository->getMediaFolderById($formFieldValue);
            return $folder;
          } else {
            return null;
          }
        }
      ))
    ;
  }

  public function getParent()
  {
    return ChoiceType::class;
  }

  public function getName()
  {
    return 'media_folder_type';
  }

  private function mediaFolderAsChoiceArray() {
    $mediaFolders = $this->repository->getMediaFoldersAsArray($this->auth->isGranted('ROLE_SUPER_ADMIN'));
    $tree = $this->buildTree($mediaFolders);
    $choices = $this->convertTreeToChoiceArray($tree);

    return $choices;
  }

  private function buildTree(array &$elements, $parentId = null) {
    $branch = array();

    foreach ($elements as &$element) {
      if ((empty($parentId) && empty($element['parent'])) || (!empty($element['parent']) &&  $element['parent']['id'] == $parentId)) {
        $children = $this->buildTree($elements, $element['id']);
        if ($children) {
            $element['children'] = $children;
        }
        $branch[$element['id']] = $element;
        unset($element);
      }
    }
    return $branch;
  }

  private function convertTreeToChoiceArray(array $tree) {
    $choices = array(
      'media.breadcrumb.root' => null
    );

    foreach($tree as $node) {
      $this->convertChildrenToChoiceArray($node, $choices, '&nbsp&nbsp;&nbsp;');
    }

    return $choices;
  }

  //TODO use ascii characters to draw a tree structure?
  private function convertChildrenToChoiceArray(array $treeNode, array &$choices, $offset = '') {
    $folderOffset = substr_replace($offset, '↳&nbsp;', strlen($offset), 0);
    $choices[$folderOffset . $treeNode['name']] = $treeNode['id'];

    if (isset($treeNode['children'])) {
      foreach($treeNode['children'] as $childNode) {
        $this->convertChildrenToChoiceArray($childNode, $choices, $offset . '&nbsp;&nbsp;&nbsp;');
      }
    }
  }

}
